<?php
// $Id: pnadminapi.php,v 1.9 2002/11/03 15:59:27 schup Exp $
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html

// <security msg="check for direct call">
if (!function_exists('pnModGetVar'))
{
   die ("You can't access this file directly...");
}
// </security>

// <change msg="include global language file of this module">
$modinfo = pnModGetInfo(pnModGetIDFromName('postguestbook'));
$currentlang = pnUserGetLang();
include_once("modules/$modinfo[directory]/pnlang/$currentlang/global.php");
// </change>

/**
* class that holds info for configuration variables
*/
class ConfigVar
{
   var $caption;
   var $name;
   var $default_value;
   var $validate;
   var $valid_values;
   var $width;
   var $maxlength;

   function ConfigVar($name, $caption)
   {
      $this->caption = $caption;
      $this->name = $name;      
      if (isset($name)) {
         // -> name not set: show only caption as info-line
         $this->default_value = pnModGetVar('postguestbook', $name);
         if (! isset($this->default_value)) {
         	$this->default_value = '0';
         }
      }      
      $this->width = 25;
      $this->maxlength = 64;
   }

}

/**
* creation of Configuartion Variables Info Array
*/
function postguestbook_adminapi_getconfigvars()
{
   static $ourConfigVarArray;
   if (! isset($ourConfigVarArray))
   {
      $ourConfigVarArray = array();
      $configVar = new ConfigVar('entries_per_page', _PGB_ENTRIESPERPAGE);
      $configVar->validate = "/^[0-9]+$/";
      array_push($ourConfigVarArray, $configVar);

      $configVar = new ConfigVar('style', _PGB_DISPLAYSTYLE);

      // <feature msg="get available styles">
      $available_styles = array();
      $modinfo = pnModGetInfo(pnModGetIDFromName('postguestbook'));
      $style_dir = "modules/$modinfo[directory]/styles";

      if (!is_dir($style_dir))
      {
         die("Styledir not accessible");
      }

      $dirHandle = opendir($style_dir);
      while ($entry = readdir($dirHandle))
      {
         if ($entry != 'CVS' && $entry != '.' && $entry != '..' &&
             is_dir("$style_dir/$entry"))
         {
            array_push($available_styles, array('id' => $entry, 'name' => $entry));
         }
      }
      closedir($dirHandle);
      $configVar->valid_values = $available_styles;
      // </feature>

      array_push($ourConfigVarArray, $configVar);

      // <feature msg="disable pnAuthentication for new posts">
      $configVar = new ConfigVar('sign_check_by_cms',  _PGB_POST_ALLOW_CHECK);
      $authentication = array(array('id' => '0', 'name' => _PGB_POST_ALLOW_EVERYONE),
                     array('id' => '1', 'name' => _PGB_POST_ALLOW_PNAUTHENTICATION));

      $configVar->valid_values = $authentication;
      array_push($ourConfigVarArray, $configVar);
      // </feature>

      $configVar = new ConfigVar('enable_html', _PGB_ENABLE_HTML);
      $yesno = array(array('id' => '0', 'name' => _PGB_NO),
                     array('id' => '1', 'name' => _PGB_YES));

      $configVar->valid_values = $yesno;
      array_push($ourConfigVarArray, $configVar);

      $configVar = new ConfigVar('enable_bbcode', _PGB_ENABLE_BBCODE);
      $configVar->valid_values = $yesno;
      array_push($ourConfigVarArray, $configVar);

		// for some reason setting enable_autolink doesn't work anymore
      $configVar = new ConfigVar('enable_autolink', _PGB_ENABLE_AUTOLINK);
      $configVar->valid_values = $yesno;
      array_push($ourConfigVarArray, $configVar);

      $configVar = new ConfigVar('notify', _PGB_NOTIFY);
      $configVar->valid_values = $yesno;
      array_push($ourConfigVarArray, $configVar);

      $configVar = new ConfigVar('notify_email',    _PGB_NOTIFY_EMAIL);
      $configVar->width=40;
      array_push($ourConfigVarArray, $configVar);

      $configVar = new ConfigVar('notify_subject',  _PGB_NOTIFY_SUBJECT);
      $configVar->width=40;
      array_push($ourConfigVarArray, $configVar);

      $configVar = new ConfigVar('notify_message',  _PGB_NOTIFY_MESSAGE);
      $configVar->width=40;
      array_push($ourConfigVarArray, $configVar);

      $configVar = new ConfigVar('notify_from',     _PGB_NOTIFY_FROM);
      $configVar->width=40;
      array_push($ourConfigVarArray, $configVar);
		
		// enable option only, if more than one smarty versions are present in the distribution
		// @todo previously loaed smarty versions 
		if (file_exists("modules/$modinfo[directory]/smarty/Smarty.class.php") &&
			 file_exists("modules/$modinfo[directory]/smarty201/Smarty.class.php"))
		{
			$configVar = new ConfigVar('use_smarty_version', _PGB_SMARTY_VERSION);
			$smver = array(array('id' => 'old', 'name' => _PGB_SMARTY_OLD),
         	            array('id' => 'new', 'name' => _PGB_SMARTY_NEW));
		}

      $configVar->valid_values = $smver;
      array_push($ourConfigVarArray, $configVar);		
   }	
   return $ourConfigVarArray;

}

/**
 * delete a postguestbook item
 * @param $args['id'] ID of the item
 * @returns bool
 * @return true on success, false on failure
 */
function postguestbook_adminapi_delete($args)
{
   // Get arguments from argument array - all arguments to this function
   // should be obtained from the $args array, getting them from other
   // places such as the environment is not allowed, as that makes
   // assumptions that will not hold in future versions of PostNuke
   extract($args);

   // Argument check - make sure that all required arguments are present,
   // if not then set an appropriate error message and return
   if (!isset($id))
   {
      pnSessionSetVar('errormsg', _MODARGSERROR);
      return false;
   }

   // Load API.  Note that this is loading the user API in addition to
   // the administration API, that is because the user API contains
   // the function to obtain item information which is the first thing
   // that we need to do.  If the API fails to load an appropriate error
   // message is posted and the function returns
   if (!pnModAPILoad('postguestbook', 'user'))
   {
      $output->Text(_LOADFAILED);
      return $output->GetOutput();
   }

   // The user API function is called.  This takes the item ID which
   // we obtained from the input and gets us the information on the
   // appropriate item.  If the item does not exist we post an appropriate
   // message and return
   $item = pnModAPIFunc('postguestbook', 'user', 'get', array('id' => $id));

   if ($item == false)
   {
         $output->Text(_PGB_NOSUCHITEM);
         return $output->GetOutput();
   }

   // Security check - important to do this as early on as possible to
   // avoid potential security holes or just too much wasted processing.
   // However, in this case we had to wait until we could obtain the item
   // name to complete the instance information so this is the first
   // chance we get to do the check
   if (!pnSecAuthAction(0, 'postguestbook::Item', "$item[name]::$id", ACCESS_DELETE))
   {
      pnSessionSetVar('errormsg', _TEMPLATENOAUTH);
      return false;
   }

   // Get datbase setup - note that both pnDBGetConn() and pnDBGetTables()
   // return arrays but we handle them differently.  For pnDBGetConn()
   // we currently just want the first item, which is the official
   // database handle.  For pnDBGetTables() we want to keep the entire
   // tables array together for easy reference later on
   list($dbconn) = pnDBGetConn();
   $pntable = pnDBGetTables();

   // It's good practice to name the table and column definitions you
   // are getting - $table and $column don't cut it in more complex
   // modules
   $guestbook_table = $pntable['postguestbook_guestbook'];
   $guestbook_column = &$pntable['postguestbook_guestbook_column'];
   // Delete the item - the formatting here is not mandatory, but it does
   // make the SQL statement relatively easy to read.  Also, separating
   // out the sql statement from the Execute() command allows for simpler
   // debug operation if it is ever needed
   $sql = "DELETE FROM $guestbook_table
         WHERE $guestbook_column[id] = " . pnVarPrepForStore($id);
   //echo $sql;
   $dbconn->Execute($sql);

   // Check for an error with the database code, and if so set an
   // appropriate error message and return
   if ($dbconn->ErrorNo() != 0)
   {
      pnSessionSetVar('errormsg', _DELETEFAILED);
      return false;
   }

   // Let any hooks know that we have deleted an item.  As this is a
   // delete hook we're not passing any extra info
   pnModCallHooks('item', 'delete', $id, '');

   // Let the calling process know that we have finished successfully
   return true;
}

/**
 * update a Guestbook entry
 */
function postguestbook_adminapi_update($args)
{
   // Get arguments from argument array - all arguments to this function
   // should be obtained from the $args array, getting them from other
   // places such as the environment is not allowed, as that makes
   // assumptions that will not hold in future versions of PostNuke
   extract($args);

   // Argument check - make sure that all required arguments are present,
   // if not then set an appropriate error message and return
   if ((!isset($id)) ||
      (!isset($name)) ||
      (!isset($message)))
   {
      pnSessionSetVar('errormsg', _MODARGSERROR . "[id=$id, name=$name, message=$message]");
      return false;
   }

   // Load API.  Note that this is loading the user API in addition to
   // the administration API, that is because the user API contains
   // the function to obtain item information which is the first thing
   // that we need to do.  If the API fails to load an appropriate error
   // message is posted and the function returns
   if (!pnModAPILoad('postguestbook', 'user'))
   {
      $output->Text(_LOADFAILED);
      return $output->GetOutput();
   }

   // The user API function is called.  This takes the item ID which
   // we obtained from the input and gets us the information on the
   // appropriate item.  If the item does not exist we post an appropriate
   // message and return
   $item = pnModAPIFunc('postguestbook', 'user', 'get', array('id' => $id));

   if ($item == false)
   {
      $output->Text(_PGB_SUCHITEM);
      return $output->GetOutput();
   }

   // Security check - important to do this as early on as possible to
   // avoid potential security holes or just too much wasted processing.
   // However, in this case we had to wait until we could obtain the item
   // name to complete the instance information so this is the first
   // chance we get to do the check
   if (!pnSecAuthAction(0, 'postguestbook::Item', "::$id", ACCESS_ADMIN)) {
      pnSessionSetVar('errormsg', _PGB_NOAUTH);
      return false;
   }

   list($dbconn) = pnDBGetConn();
   $pntable = pnDBGetTables();

   // It's good practice to name the table and column definitions you
   // are getting - $table and $column don't cut it in more complex
   // modules
   $guestbook_table = $pntable['postguestbook_guestbook'];
   $guestbook_column = &$pntable['postguestbook_guestbook_column'];
   // Delete the item - the formatting here is not mandatory, but it does
   // make the SQL statement relatively easy to read.  Also, separating
   // out the sql statement from the Execute() command allows for simpler
   // debug operation if it is ever needed
   // don't update id, ip and timestamp

   if (empty($members))
   {
      $members= "";
   }

   // don't update owner of guestbook_entry
   // $guestbook_column[owner_uid] = ". pnVarPrepForStore($owner_uid. "',
   $sql = "UPDATE $guestbook_table
         SET
               $guestbook_column[name] = '". pnVarPrepForStore($name) ."',
               $guestbook_column[email]  = '" . pnVarPrepForStore($email) ."',
               $guestbook_column[message]  = '" . pnVarPrepForStore($message) ."',
               $guestbook_column[comment] = '" . pnVarPrepForStore($comment)."',
               $guestbook_column[homepage] = '" . pnVarPrepForStore($homepage) ."',
               $guestbook_column[members] = '" . pnVarPrepForStore($members) ."',
               $guestbook_column[private_msg] = '" . pnVarPrepForStore($private_msg) ."',
               $guestbook_column[location] = '" . pnVarPrepForStore($location) ."',
               $guestbook_column[mood] = '" . pnVarPrepForStore($mood) ."',
               $guestbook_column[user1] = '" . pnVarPrepForStore($user1) ."',
               $guestbook_column[user2] = '" . pnVarPrepForStore($user2) ."',
               $guestbook_column[user3] = '" . pnVarPrepForStore($user3) ."',
               $guestbook_column[user4] = '" . pnVarPrepForStore($user4) ."',
               $guestbook_column[user5] = '" . pnVarPrepForStore($user5) ."',
               $guestbook_column[disable_html] = '" . pnVarPrepForStore($disable_html) . "',
               $guestbook_column[disable_bbcode] = '" . pnVarPrepForStore($disable_bbcode) . "',
               $guestbook_column[disable_autolinks] = '" . pnVarPrepForStore($disable_autolinks) . "',
               $guestbook_column[pn_uid] = '" . pnVarPrepForStore($pn_uid) . "'
         WHERE $guestbook_column[id] = " . pnVarPrepForStore($id);

   $dbconn->Execute($sql);


   // Check for an error with the database code, and if so set an
   // appropriate error message and return
   if ($dbconn->ErrorNo() != 0)
   {
      pnSessionSetVar('errormsg', _PGB_UPDATE_FAILED . ": [$sql]");
      return false;
   }

   // Let the calling process know that we have finished successfully
   return true;
}

/**
 * update the guestbook configuration
 */
function postguestbook_adminapi_updateconfig($args)
{
   if (!pnSecAuthAction(0, 'postguestbook::Item', "::", ACCESS_ADMIN))
   {
      $output->Text(_PGB_NOAUTH);
      return $output->GetOutput();
   }

   $configVars = postguestbook_adminapi_getconfigvars();
   foreach($configVars as $configVar)
   {
      $value = pnVarCleanFromInput($configVar->name);      
      if (isset($value))
      {
			// <feature msg="check value, if validated specified">
			if (isset($configVar->validate))
			{
				if (!preg_match($configVar->validate, $value))
				{
					// invalid value
					//die(" $configVar->name=[$value]");
					pnSessionSetVar('errormsg', _PGB_INVALID_VALUE . " $configVar->caption=[$value]");
					pnRedirect(pnModURL('postguestbook', 'admin', 'view'));
					return false;
				}
			}
			// </feature>
			// <feature msg="check style">
			if ($configVar->name == "style")
			{
				$modinfo = pnModGetInfo(pnModGetIDFromName('postguestbook'));
				$style_dir = "modules/$modinfo[directory]/styles/" . $value;
	
				if (!is_dir($style_dir))
				{
					pnSessionSetVar('errormsg', _PGB_INVALID_VALUE . " styledir not accessible");
				}
			}
			// </feature> 
      } 
      else
      {
      		$value = '0';
      }
      pnModSetVar('postguestbook', $configVar->name, $value);
   }
	
   // <feature msg="clear smarty compiled templates">
   $modinfo = pnModGetInfo(pnModGetIDFromName('postguestbook'));
	// smarty optional
	if (file_exists("modules/$modinfo[directory]/smarty_view.php")) 
	{
		include_once("modules/$modinfo[directory]/smarty_view.php");
		$smarty = postguestbook_view_create_smarty();
		@$smarty->clear_compiled_tpl();
	}
   // </feature>

   // Confirm authorisation code.  This checks that the form had a valid
   // authorisation code attached to it.  If it did not then the function will
   // proceed no further as it is possible that this is an attempt at sending
   // in false data to the system
   /*
   if (!pnSecConfirmAuthKey())
   {
      pnSessionSetVar('errormsg', _BADAUTHKEY);
      pnRedirect(pnModURL('postguestbook', 'user', 'view'));
      return true;
   }
   */
   // Success
   pnSessionSetVar('statusmsg', _PGB_CONFIG_UPDATED);

   // This function generated no output, and so now it is complete we redirect
   // the user to an appropriate page for them to carry on their work
   pnRedirect(pnModURL('postguestbook', 'admin', 'view'));

   // Return
   return true;
}
?>
